# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)
logger = logging.getLogger("driverng.drivers.scanlab")


class ScanlabSegmentH(HeaderSegment):
    emetteur = HprimString(5, max_length=15)
    recepteur = HprimString(10, max_length=15)
    id_traitement = HprimString(12, default="P")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class ScanlabSegmentP(Segment):
    no_patient = HprimString(3)
    nom = HprimString((6, 1))
    prenom = HprimString((6, 2))
    nomjf = HprimString(7)
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")

    # M/F/U
    sexe = HprimString(9)
    service = HprimString(26)
    physician_id = HprimString(14, max_length=6)

    orders = HprimForeign("OBR", "patient")


class ScanlabSegmentOBR(Segment):
    # Pour le message de résultat ça dépend de la version du SCANLAB
    old_sample_id = HprimString((3, 1))
    sample_id = HprimString((3, 2), max_length=14)

    tests = HprimList(5, basefield=HprimMultiple(
        0, code=HprimString(1)))

    # S: Stat
    # R: Routine
    # urgence = HprimString(6, max_length=1)
    serotheque = HprimString(6)

    collection_date = HprimDate(8)
    specimen_code = HprimString(16, max_length=3)

    results = HprimForeign("OBX", "order")


class ScanlabSegmentOBX(Segment):

    # code profile
    test = HprimString(4)

    # id_scanlab
    id_scanlab = HprimString(6)

    # flag serotheque
    flag = HprimString(9)

    # Contenant
    # selon le format suivant :
    # Congelat.:CXX;Rayon:RXX;Portoir:XXXXX;Repere:X;
    # le congelat est un identifiant unique
    contenant = HprimString(13)

    # date du contenant
    date = HprimDate(14)


class ScanlabSegmentL(Segment):

    # terminaison
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "scanlab",
        "label": "Scanlab",
    }

    SEGMENT_CLASS_MAP = {
        "H": ScanlabSegmentH,
        "P": ScanlabSegmentP,
        "OBR": ScanlabSegmentOBR,
        "OBX": ScanlabSegmentOBX,
        "L": ScanlabSegmentL,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "scanlab_envoyer_no_serotheque",
            label="Envoyer le numéro sérotheque dans le champ OBR.6"),
        AstmMessageV2.Option(
            "scanlab_code_connexion_dms_tracking",
            label="Code de la connexion Dms Tracking",
            default=""),
        AstmMessageV2.BooleanOption(
            "scanlab_transcodage_rack_dms_tracking",
            label="Transcoder le rack du DMS Tracking au format Scanlab"),
        AstmMessageV2.IntegerOption(
            "scanlab_sequence_rack",
            label="Sequence inclue dans le rack"),
    ]

    TRANSCODE_POSITION_RACK = {
        "01": "48",
        "02": "44",
        "03": "40",
        "04": "36",
        "05": "32",
        "06": "28",
        "07": "24",
        "08": "20",
        "09": "16",
        "10": "12",
        "11": "08",
        "12": "04",
        "13": "47",
        "14": "43",
        "15": "39",
        "16": "35",
        "17": "31",
        "18": "27",
        "19": "23",
        "20": "19",
        "21": "15",
        "22": "11",
        "23": "07",
        "24": "03",
        "25": "46",
        "26": "42",
        "27": "38",
        "28": "34",
        "29": "30",
        "30": "26",
        "31": "22",
        "32": "18",
        "33": "14",
        "34": "10",
        "35": "06",
        "36": "02",
        "37": "45",
        "38": "41",
        "39": "37",
        "40": "33",
        "41": "29",
        "42": "25",
        "43": "21",
        "44": "17",
        "45": "13",
        "46": "09",
        "47": "05",
        "48": "01"
    }

    def __init__(self, **kwargs):
        self.envoyer_no_serotheque = kwargs.pop("scanlab_envoyer_no_serotheque", False)
        self.transcodage_rack_dms_tracking = kwargs.pop("scanlab_transcodage_rack_dms_tracking", False)
        self.sequence_rack = kwargs.pop("scanlab_sequence_rack", "")
        self.scanlab_code_connexion_dms_tracking = kwargs.pop("scanlab_code_connexion_dms_tracking", "")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = ScanlabSegmentH(type_segment="H", **kwargs)
        message.end = ScanlabSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"S": "S", "A": "A", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                no_patient=patient.patient_id,
                nom=patient.last_name,
                prenom=patient.first_name,
                nomjf=patient.maiden_name,
                date_naissance=patient.birthdate,
                sexe=patient.sex,
                service=patient.location,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    # urgence=priority,
                    specimen_code=sample.specimen,
                    collection_date=sample.collection_date,
                )
                if self.envoyer_no_serotheque and sample.info_serotheque:
                    seg_o.serotheque = "{}/{};{}".format(
                        sample.info_serotheque.portoir,
                        sample.info_serotheque.numero,
                        sample.info_serotheque.position)
                else:
                    # Envoyer le résultat accueil RACK selon le format SCANLAB
                    rack_position = ""
                    for result in sample.results:
                        if result.code.startswith("RACK"):
                            if "-" in result.value:
                                rack_position = result.value.split("-")
                                continue
                    if rack_position and not self.transcodage_rack_dms_tracking:
                        seg_o.serotheque = "P50/{};{}".format(
                            rack_position[0],
                            rack_position[1],
                        )
                    elif self.transcodage_rack_dms_tracking:
                        transcode_position_rack = self.TRANSCODE_POSITION_RACK.get(sample.rack_dms_tracking_serotheque.split("-")[-1], "")
                        if not transcode_position_rack:
                            logger.info("Pas de correspondance trouvé pour la position %s pour le rack dms_tracking %s",
                                        sample.rack_dms_tracking_serotheque.split("-")[-1],
                                        sample.rack_dms_tracking_serotheque)
                        elif transcode_position_rack and sample.sequence_scanlab:
                            seg_o.serotheque = "P48/{:05}:{}".format(
                                int(sample.sequence_scanlab),
                                transcode_position_rack
                            )
                            logger.info("Le rack %s du tube %s du module IOM de DMS est transcodé en position Scanlab %s",
                                        sample.rack_dms_tracking_serotheque,
                                        sample.sample_id,
                                        transcode_position_rack)
                for order in sample.orders:
                    if order.code == "RACK":
                        continue
                    t = seg_o.tests.add()
                    t.code = order.code

        if len(msg.patients):
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.sample_id or seg_o.old_sample_id,
                    )
                    for seg_r in seg_o.results:
                        sample.results.add(
                            code=seg_r.test,
                            value=seg_r.contenant,
                            date=seg_r.date,
                        )
        return worklist
